﻿using System;
using System.Runtime.InteropServices;

namespace CGroupBasedTargetMemoryControl.Util
{
    public static class LibCGroup
    {
        [DllImport("libcgroup.so.2")]
        public static extern int cgroup_init();

        [DllImport("libcgroup.so.2")]
        public static extern nint cgroup_new_cgroup(string name);

        [DllImport("libcgroup.so.2")]
        public static extern nint cgroup_add_controller(nint cgroup, string controller);

        [DllImport("libcgroup.so.2")]
        public static extern int cgroup_set_value_string(
            nint controller,
            string name,
            string value
        );

        [DllImport("libcgroup.so.2")]
        public static extern int cgroup_create_cgroup(nint cgroup, int ignore_ownership);

        [DllImport("libcgroup.so.2")]
        public static extern int cgroup_attach_task(nint cgroup);

        [DllImport("libcgroup.so.2")]
        public static extern void cgroup_free(nint cgroup);

        [DllImport("libcgroup.so.2")]
        public static extern unsafe int cgroup_get_all_controller_begin(
            void** handle,
            ref controller_data info
        );

        [DllImport("libcgroup.so.2")]
        public static extern unsafe int cgroup_get_all_controller_next(
            void** handle,
            ref controller_data info
        );

        [DllImport("libcgroup.so.2")]
        public static extern unsafe int cgroup_get_all_controller_end(void** handle);

        [DllImport("libcgroup.so.2", CharSet = CharSet.Ansi)]
        public static extern IntPtr cgroup_strerror(int code);

        [DllImport("libcgroup.so.2", CharSet = CharSet.Ansi)]
        public static extern unsafe int cgroup_get_procs(
            string name,
            string controller,
            out IntPtr pids, // pid_t** -> IntPtr（指向pid_t数组的指针）
            out int size // int* -> out int
        );

        public static string GetErrorString(int code)
        {
            IntPtr ptr = cgroup_strerror(code);
            return Marshal.PtrToStringAnsi(ptr)!;
        }
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct controller_data
    {
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
        public string name;
        public int hierarchy;
        public int num_cgroups;
        public int enabled;
    }
}
